import urllib.request
import json
from datetime import datetime


class SpotSDKError(Exception):
    pass

class Message(object):
  def __init__(self, raw_message):
    if not isinstance(raw_message, dict):
      raise SpotSDKError('message should be dictionary')

    self.id =            raw_message['id']
    self.type =          raw_message['messageType']
    self.battery_state = raw_message['batteryState']
    self.latitude =      raw_message['latitude']
    self.longitude =     raw_message['longitude']
    self.datetime =      datetime.utcfromtimestamp(raw_message['unixTime'])
    self.content =       self.__content(raw_message)
    self.raw =           raw_message

  NO_MESSAGE_CONTENT = 'No message content'

  def __content(self, raw_message):
    if 'messageContent' in raw_message:
      return raw_message['messageContent']
    else:
      return self.NO_MESSAGE_CONTENT


class Feed(object):
  def __init__(self, key):
    if not isinstance(key, str):
      raise SpotSDKError('key should be string')
    self.key      = key
    self.messages = []

  BASE_URL = 'https://api.findmespot.com/spot-main-web/consumer/rest-api/2.0/public/feed/'

  def __request_url(self):
    return self.BASE_URL + self.key + '/message.json'

  def first(self):
      return self.messages[0]

  def last(self):
      return self.messages[-1]

  def count(self):
      return len(self.messages)

  def collect(self):
    self.collected_at = datetime.now()

    url          = self.__request_url()
    raw_response = urllib.request.urlopen(url).read()
    response     = json.loads(raw_response)['response']

    if 'errors' in response.keys():
      raise SpotSDKError(response['errors']['error']['text'])

    raw_messages = response['feedMessageResponse']['messages']['message']
    self.__create_messages(raw_messages)

  def __create_messages(self, raw_messages):
    for raw_message in raw_messages:
        self.messages.append(Message(raw_message))



import random
import unittest
from unittest.mock import patch, mock_open
from datetime import datetime

from spot_sdk import Feed, Message, SpotSDKError

class TestMessage(unittest.TestCase):

  raw_message = {
    "@clientUnixTime": "0",
    "id": 823677113,
    "messengerId": "0-3049668",
    "messengerName": "svcatsaway",
    "unixTime": 1504560201,
    "messageType": "UNLIMITED-TRACK",
    "latitude": 44.26225,
    "longitude": -76.37656,
    "modelId": "SPOT3",
    "showCustomMsg": "Y",
    "dateTime": "2017-09-04T21:23:21+0000",
    "batteryState": "GOOD",
    "hidden": 0,
    "altitude": 74
  }

  def test_creating_message(self):
    message = Message(self.raw_message)

    self.assertTrue(message.id == 823677113)
    self.assertTrue(message.type == "UNLIMITED-TRACK")
    self.assertTrue(message.battery_state == "GOOD")
    self.assertTrue(message.latitude == 44.26225)
    self.assertTrue(message.longitude == -76.37656)
    self.assertTrue(message.datetime == datetime(2017, 9, 4, 21, 23, 21))
    self.assertTrue(message.content == Message.NO_MESSAGE_CONTENT)
    self.assertTrue(message.raw == self.raw_message)

  def test_with_content(self):
    raw_message = self.raw_message
    content = "The cats are purring"
    raw_message['messageContent'] = content
    message = Message(raw_message)

    self.assertTrue(message.content == content)


class TestFeed(unittest.TestCase):
  success_json_reponse = open('spot_sdk/sample_success_response.json', 'r').read()
  mock_success_urlopen = mock_open(read_data=success_json_reponse)
  @patch('urllib.request.urlopen', mock_success_urlopen)
  def test_creating_feed(self):
    feed = Feed('ValidApiKey')
    self.assertTrue(feed.count() == 2)
    self.assertTrue(feed.first().id == 823675096)
    self.assertTrue(feed.last().id == 823674967)
    self.assertTrue(isinstance(feed.messages, list))

  error_json_reponse = open('spot_sdk/sample_error_response.json', 'r').read()
  mock_error_urlopen = mock_open(read_data=error_json_reponse)
  @patch('urllib.request.urlopen', mock_error_urlopen)
  def test_incorrect_api_key(self):
    with self.assertRaises(SpotSDKError):
      Feed('InvalidApiKey')

if __name__ == '__main__':
    unittest.main()
